/*
 * Copyright (c) 2002-2009 syntevo GmbH. All Rights Reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  o Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  o Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 *  o Neither the name of syntevo GmbH nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.syntevo.q.gui.layout;

import java.io.*;
import javax.swing.*;

/**
 * This is a convenience class to make using the QTableGridLayout easier.
 * <br>
 * First, you need to define the grid using the addColumn* and addRow* methods.
 * Then you add the components or editors using the addComponent or addEditor methods.
 *
 * @author Thomas Singer
 */
public class TableGridLayoutFactory {

	// Static =================================================================

	public static boolean isEDTorEDTLike() {
		if (SwingUtilities.isEventDispatchThread()) {
			return true;
		}

		final StringWriter stringWriter = new StringWriter();
		new Exception().printStackTrace(new PrintWriter(stringWriter));
		final String stackTrace = stringWriter.toString();
		return stackTrace.indexOf("\tat com.sun.java.accessibility") >= 0;
	}

	// Fields =================================================================

	private final TableGridLayout layout;
	private final JComponent container;

	// Setup ==================================================================

	public TableGridLayoutFactory(JComponent container) {
		this(container, false);
	}

	private TableGridLayoutFactory(JComponent container, boolean ignoreInvisibleComponents) {
		if (container == null) {
			container = new JPanel();
			container.setFocusable(false);
		}

		this.container = container;
		layout = new TableGridLayout(container, ignoreInvisibleComponents);
		container.setLayout(layout);

		assertEventDispatchThread();
	}

	// Accessing ==============================================================

	public final JComponent getContainer() {
		return container;
	}

	public final int getLastColumnIndex() {
		return layout.getLastColumnIndex();
	}

	public final int getLastRowIndex() {
		return layout.getLastRowIndex();
	}

	public final int addRowNoStretch() {
		return addRowNoStretch(0);
	}

	public final int addRowNoStretch(int height) {
		return addRowNoStretch(height, null);
	}

	public final int addRowNoStretch(int height, TableGridGroup rowGroup) {
		return addRow(height, TableGridStretchValue.NO_STRETCH, rowGroup);
	}

	public final int addRowStretchOnResize() {
		return addRowStretchOnResize(0);
	}

	public final int addRowStretchOnResize(int height) {
		return addRowStretchOnResize(height, null);
	}

	public int addRowStretchOnResize(int height, TableGridGroup rowGroup) {
		return addRow(height, TableGridStretchValue.STRETCH_ON_RESIZE, rowGroup);
	}

	public final int addRowStretchIfRequired() {
		return addRowStretchIfRequired(0);
	}

	public final int addRowStretchIfRequired(int height) {
		return addRowStretchIfRequired(height, null);
	}

	public final int addColumnNoStretch() {
		return addColumnNoStretch(0);
	}

	public final int addColumnNoStretch(int width) {
		return addColumnNoStretch(width, null);
	}

	public final int addColumnNoStretch(int width, TableGridGroup columnGroup) {
		return addColumn(width, TableGridStretchValue.NO_STRETCH, columnGroup);
	}

	public final int addColumnStretchOnResize() {
		return addColumnStretchOnResize(0);
	}

	public final int addColumnStretchOnResize(int minWidth) {
		return addColumnStretchOnResize(minWidth, null);
	}

	public final int addColumnStretchOnResize(int minWidth, TableGridGroup columnGroup) {
		return addColumn(minWidth, TableGridStretchValue.STRETCH_ON_RESIZE, columnGroup);
	}

	public final int addColumnStretchIfRequired() {
		return addColumnStretchIfRequired(0);
	}

	public final int addColumnStretchIfRequired(int minWidth) {
		return addColumnStretchIfRequired(minWidth, null);
	}

	public final int addColumnStretchIfRequired(int minWidth, TableGridGroup columnGroup) {
		return addColumn(minWidth, TableGridStretchValue.STRETCH_IF_REQUIRED, columnGroup);
	}

	public final void addComponent(JComponent component, int row, int column) {
		addComponent(component, row, column, 1, 1, true, true, true, true);
	}

	public final void addComponent(JComponent component,
	                               int row, int column, int height, int width) {
		addComponent(component, row, column, height, width, true, true, true, true);
	}

	public final void addComponent(JComponent component, int row, int column,
	                               boolean alignTop, boolean alignLeft, boolean alignBottom, boolean alignRight) {
		addComponent(component, row, column, 1, 1, alignTop, alignLeft, alignBottom, alignRight);
	}

	public final void addComponent(JComponent component,
	                               int row, int column, int height, int width,
	                               boolean alignTop, boolean alignLeft, boolean alignBottom, boolean alignRight) {
		assertEventDispatchThread();
		container.add(component, new TableGridConstraints(row, column, height, width, alignTop, alignLeft, alignBottom, alignRight));
	}

	/**
	 * Use for debugging the layout of a certain panel.
	 *
	 * @deprecated this method only marked as deprecated to let IDEs highlight it, so you don't forget to remove it
	 */
	public final void setDebug() {
		TableGridLayout.setDebugInstance(layout);
	}

	// Utils ==================================================================

	private int addRow(int height, TableGridStretchValue stretchValue, TableGridGroup rowGroup) {
		assertEventDispatchThread();
		return layout.addRow(height, stretchValue, rowGroup);
	}

	private int addRowStretchIfRequired(int height, TableGridGroup rowGroup) {
		return addRow(height, TableGridStretchValue.STRETCH_IF_REQUIRED, rowGroup);
	}

	private int addColumn(int minWidth, TableGridStretchValue stretchValue, TableGridGroup columnGroup) {
		assertEventDispatchThread();
		return layout.addColumn(minWidth, stretchValue, columnGroup);
	}

	private static void assertEventDispatchThread() {
		if (!isEDTorEDTLike()) {
			throw new IllegalStateException("No EDT");
		}
	}
}
